/** @file HL_sys_main.c 
*   @brief Application main file
*   @date 11-Dec-2018
*   @version 04.07.01
*
*   This file contains an empty main function,
*   which can be used for the application.
*/

/* 
* Copyright (C) 2009-2018 Texas Instruments Incorporated - www.ti.com  
* 
* 
*  Redistribution and use in source and binary forms, with or without 
*  modification, are permitted provided that the following conditions 
*  are met:
*
*    Redistributions of source code must retain the above copyright 
*    notice, this list of conditions and the following disclaimer.
*
*    Redistributions in binary form must reproduce the above copyright
*    notice, this list of conditions and the following disclaimer in the 
*    documentation and/or other materials provided with the   
*    distribution.
*
*    Neither the name of Texas Instruments Incorporated nor the names of
*    its contributors may be used to endorse or promote products derived
*    from this software without specific prior written permission.
*
*  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
*  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
*  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
*  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
*  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
*  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
*  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
*  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
*  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
*  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
*  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*
*/


/* USER CODE BEGIN (0) */
/* USER CODE END */

/* Include Files */

#include "HL_sys_common.h"

/* USER CODE BEGIN (1) */
#include "FreeRTOS.h"
#include "os_task.h"
#include "HL_spi.h"
#include "HL_reg_spi.h"
#include "HL_mibspi.h"
#include "HL_gio.h"
#include "HL_het.h"
/* USER CODE END */

/** @fn void main(void)
*   @brief Application main function
*   @note This function is empty by default.
*
*   This function is called after startup.
*   The user can use this function to implement the application.
*/

/* USER CODE BEGIN (2) */
#define bit_test(byte,bit) (byte & (1 << bit)) // test for bit set

#define SPI3_CS0 1
#define HIGH 1
#define LOW 0

spiDAT1_t dataconfig1_t;

void wait_secons(float32 time) {
    uint32 k_delay;

    k_delay = (uint32) (-3.6932 + 8.33333e6 * time) * 176 / 100; //Ecuacion para el Retardo, calculo fisico

    while (k_delay) {
        k_delay--;
    };
}

int Ready_25LC(void) {

  uint16 buffer_status[2]={0x03,0x00};                  // Initial Instruction

  gioSetDirection(spiPORT5, 0x3);

  gioSetBit(spiPORT3, SPI3_CS0, LOW);


  wait_secons(100e-6);

  spiTransmitData(spiREG3, &dataconfig1_t, 1, (uint16*) buffer_status);

  spiReceiveData(spiREG3, &dataconfig1_t, 1, (uint16*) buffer_status);

  gioSetBit(spiPORT3, SPI3_CS0, HIGH);

  return (!bit_test(buffer_status[0], 0));

}

uint16 Read_25LC(int address) {
  uint16 read_data[2];

  uint16 buffer_read[4];

  while (!Ready_25LC());

  /*Load Instruction and address and data*/

  buffer_read[0] = (uint16) 0x03;

  buffer_read[1] = (uint16) address >> 8;

  buffer_read[2] = (uint16) address;

  gioSetBit(spiPORT3, SPI3_CS0, LOW);

  spiTransmitData(spiREG3, &dataconfig1_t, 3, buffer_read);

  spiReceiveData(spiREG3, &dataconfig1_t, 1, (uint16*) read_data);

  gioSetBit(spiPORT3, SPI3_CS0, HIGH);

  return (read_data[0]);

}


void Write_25LC(int address, uint8 write_data) {

  uint16 buffer_write[4]={0x06,0x00,0x00,0x00}; // Initial Instruction

  while (!Ready_25LC());

  gioSetBit(spiPORT3, SPI3_CS0, LOW);

  spiTransmitData(spiREG3, &dataconfig1_t, 1, buffer_write);

  gioSetBit(spiPORT3, SPI3_CS0, HIGH);

  /*Load Instruction and address and data*/

  buffer_write[0] = (uint16) 0x02;

  buffer_write[1] = (uint16) address >> 8;

  buffer_write[2] = (uint16) address;

  buffer_write[3] = (uint16) write_data;

  /*Low CS0 and Transmit Data SPI to 25LC256 and High CS0*/

  gioSetBit(spiPORT3, SPI3_CS0, LOW);

  spiTransmitData(spiREG3, &dataconfig1_t, 4, buffer_write);


  gioSetBit(spiPORT3, SPI3_CS0, HIGH);

}

void vTaskCode( void * pvParameters )
 {
    int i=0;
    int k=0;

    spiInit();
    gioInit();
    hetInit();
    _enable_IRQ();


    dataconfig1_t.CS_HOLD = TRUE;
    dataconfig1_t.CSNR = CS_1;
    //dataconfig1_t.WDEL = TRUE;
    dataconfig1_t.WDEL = FALSE;
    dataconfig1_t.DFSEL = SPI_FMT_0;

     for( ; i<10; i++ )
     {
#if 1
         Write_25LC(0, 0x35); // Task code goes here.
         if (Read_25LC(0)!=0x35)
         {
            k++;
         }
#else
    uint16 eeprom_byteread[4] = {0x03, 0x00, 0x00, 0x00}; // read from address 0x0000 (last byte is dummy, placeholder for result
    uint16 Result25LC256[5] = { 0x00, 0x00, 0x00, 0x00, 0x00 };
    spiDAT1_t dataconfig1_t;

    dataconfig1_t.CS_HOLD = FALSE;
    dataconfig1_t.WDEL = TRUE;
    dataconfig1_t.DFSEL = SPI_FMT_0;
    dataconfig1_t.CSNR = CS_1;

    spiTransmitAndReceiveData(spiREG3, &dataconfig1_t, 4, eeprom_byteread,Result25LC256);
#endif
     }
 }
/* USER CODE END */

int main(void)
{
/* USER CODE BEGIN (3) */
    static uint8_t ucParameterToPass;
    TaskHandle_t xHandle = NULL;

        // Create the task, storing the handle.  Note that the passed parameter ucParameterToPass
        // must exist for the lifetime of the task, so in this case is declared static.  If it was just an
        // an automatic stack variable it might no longer exist, or at least have been corrupted, by the time
        // the new task attempts to access it.
        xTaskCreate( vTaskCode, "NAME", configMINIMAL_STACK_SIZE, &ucParameterToPass, tskIDLE_PRIORITY, &xHandle );
        configASSERT( xHandle );

        // Use the handle to delete the task.
  //      if( xHandle != NULL )
//        {
  //          vTaskDelete( xHandle );
    //    }
        vTaskStartScheduler();

/* USER CODE END */

    return 0;
}


/* USER CODE BEGIN (4) */
/* USER CODE END */
